@@ -13,13 +13,12 @@ Github: superolmo
 
 """
 
-__version__ = '0.1'
-__author__ = 'Claudio Olmi'
-
-import requests 
+from typing import List
+import requests
 import time
-from os import path as osp
+from os import device_encoding, path as osp
 from os import makedirs
+from os import remove
 
 # Timing xtream json downloads
 from timeit import default_timer as timer, timeit
@@ -29,6 +28,14 @@ import json
 # used for URL validation
 import re
 
+try:
+    from pyxtream.rest_api import FlaskWrap
+    USE_FLASK = True
+except:
+    USE_FLASK = False
+
+from pyxtream.progress import progress
+
 class Channel():
     stream_type = ""
     logo_path = ""
@@ -84,7 +91,8 @@ class Channel():
             elif stream_type == "movie":
                 stream_extension = stream_info['container_extension']
 
-            self.url = "http://mega.test25.in:80/{}/{}/{}/{}.{}".format(
+            self.url = "{}/{}/{}/{}/{}.{}".format(
+                xtream.server,
                 stream_info['stream_type'],
                 xtream.authorization['username'],
                 xtream.authorization['password'],
@@ -96,6 +104,15 @@ class Channel():
             if not xtream.validateURL(self.url):
                 print("{} - Bad URL? `{}`".format(self.name, self.url))
 
+    def export_json(self):
+        jsondata = {}
+
+        jsondata['url'] = self.url
+        jsondata.update(self.raw)
+        jsondata['logo_path'] = self.logo_path
+
+        return jsondata
+
 class Group():
     def __init__(self, group_info: dict, stream_type: str):
         # Raw JSON Group
@@ -135,7 +152,8 @@ class Episode():
         self.logo_path = xtream.getLogoLocalPath(self.logo)
         
 
-        self.url = "http://mega.test25.in:80/series/{}/{}/{}.{}".format(
+        self.url = "{}/series/{}/{}/{}.{}".format(
+            xtream.server,
             xtream.authorization['username'],
             xtream.authorization['password'],
             self.id,
@@ -163,6 +181,14 @@ class Serie():
         self.youtube_trailer = series_info['youtube_trailer']
         self.genre = series_info['genre']
 
+    def export_json(self):
+        jsondata = {}
+
+        jsondata.update(self.raw)
+        jsondata['logo_path'] = self.logo_path
+
+        return jsondata
+
 class Season():
     def __init__(self, name):
         self.name = name
@@ -210,13 +236,14 @@ class XTream():
         self.username = provider_username
         self.password = provider_password
         self.name = provider_name
+        self.cache_path = cache_path
 
         # if the cache_path is specified, test that it is a directory
-        if cache_path != "":
-            if osp.isdir(cache_path):
-                self.cache_path = cache_path
-            else:
+        if self.cache_path != "":
+            # If the cache_path is not a directory, clear it
+            if not osp.isdir(self.cache_path):
                 print("Cache Path is not a directory, using default '~/.xtream-cache/'")
+                self.cache_path == ""
         
         # If the cache_path is still empty, use default
         if self.cache_path == "":
@@ -226,6 +253,124 @@ class XTream():
 
         self.authenticate()
 
+        if USE_FLASK:
+            self.flaskapp = FlaskWrap('pyxtream', self)
+            self.flaskapp.start()
+
+    def search_stream(self, keyword: str, return_type: str = "LIST") -> List:
+        """Search for streams
+
+        Args:
+            keyword (str): Keyword to search for. Supports REGEX
+            return_type (str, optional): Output format, 'LIST' or 'JSON'. Defaults to "LIST".
+
+        Returns:
+            List: List with all the results, it could be empty. Each result
+        """
+
+        search_result = []
+        
+        regex = re.compile(keyword)
+
+        print("Checking {} movies".format(len(self.movies)))
+        for stream in self.movies:
+            if re.match(regex, stream.name) is not None:
+                search_result.append(stream.export_json())
+
+        print("Checking {} channels".format(len(self.channels)))
+        for stream in self.channels:
+            if re.match(regex, stream.name) is not None:
+                search_result.append(stream.export_json())
+
+        print("Checking {} series".format(len(self.series)))
+        for stream in self.series:
+            if re.match(regex, stream.name) is not None:
+                search_result.append(stream.export_json())
+
+        if return_type == "JSON":
+            if search_result != None:
+                print("Found {} results `{}`".format(len(search_result),keyword))
+                return json.dumps(search_result, ensure_ascii=False)
+        else:
+            return search_result
+
+    def download_video(self, stream_id: int):
+
+        url = ""
+        filename = ""
+        for stream in self.movies:
+            if stream.id == stream_id:
+                url = stream.url
+                fn = "{}.{}".format(self.slugify(stream.name),stream.raw['container_extension'])
+                filename = osp.join(self.cache_path,fn)
+
+        if url != "":
+            if not osp.isfile(filename):
+                if not self.download_video_impl(url,filename):
+                    return "Error"
+
+        return filename
+
+
+    def download_video_impl(self, url: str, fullpath_filename: str) -> bool:
+        """Download a stream
+
+        Args:
+            url (str): Complete URL of the stream
+            fullpath_filename (str): Complete File path where to save the stream.
+
+        Returns:
+            bool: True if successful, False if error
+        """
+        ret_code = False
+        mb_size = 1024*1024
+        try:
+            print("Downloading from URL `{}` and saving at `{}`".format(url,fullpath_filename))
+            
+            # Make the request to download
+            response = requests.get(url, timeout=(5), stream=True, allow_redirects=True)
+            # If there is an answer from the remote server
+            if response.status_code == 200:
+                # Get content type Binary or Text
+                content_type = response.headers.get('content-type',None)
+
+                # Get total playlist byte size
+                total_content_size = int(response.headers.get('content-length',None))
+                total_content_size_mb = total_content_size/mb_size
+
+                # Set downloaded size
+                downloaded_bytes = 0
+                
+                # Set stream blocks
+                block_bytes = int(4*mb_size)     # 4 MB
+
+                print("Ready to download {:.1f} MB file ({})".format(total_content_size_mb,total_content_size))
+                if content_type.split('/')[0] != "text":
+                    with open(fullpath_filename, "wb") as file:
+                    
+                        # Grab data by block_bytes
+                        for data in response.iter_content(block_bytes,decode_unicode=False):
+                            downloaded_bytes += block_bytes
+                            progress(downloaded_bytes,total_content_size,"Downloading")
+                            #print("{:.0f}/{:.1f} MB".format(downloaded_bytes/mb_size,total_content_size_mb))
+                            file.write(data)
+                    if downloaded_bytes < total_content_size:
+                        print("The file size is incorrect, deleting")
+                        remove(fullpath_filename)
+                    else:
+                        # Set the datatime when it was last retreived
+                        # self.settings.set_
+                        print("")
+                        ret_code = True
+                else:
+                    print("URL has a file with unexpected content-type {}".format(content_type))
+            else:
+                print("HTTP error %d while retrieving from %s!" % (response.status_code, url))
+        except Exception as e:
+            print(e)
+        
+        return ret_code
+
     def slugify(self, string: str) -> str:
         """Normalize string
 
@@ -287,35 +432,44 @@ class XTream():
             }
 
     def loadFromFile(self, filename) -> dict:
+        """Try to load the distionary from file
+
+        Args:
+            filename ([type]): File name containing the data
+
+        Returns:
+            dict: Dictionary is found and no errors, None is file does not exists
+        """
         #Build the full path
         full_filename = osp.join(self.cache_path, "{}-{}".format(
                 self.slugify(self.name), 
                 filename
         ))
 
+        if osp.isfile(full_filename):
 
-        my_data = None
-        #threshold_time = time.mktime(time.gmtime(60*60*8))   # 8 hours
-        threshold_time = 60*60*8
-
-        # Get the enlapsed seconds since last file update
-        diff_time = time.time() - osp.getmtime(full_filename)
-        # If the file was updated less than the threshold time, 
-        # it means that the file is still fresh, we can load it.
-        # Otherwise skip and return None to force a re-download
-        if threshold_time > diff_time:
-            # Load the JSON data
-            try:
-                with open(full_filename,mode='r',encoding='utf-8') as myfile:
-                    #my_data = myfile.read()
-                    my_data = json.load(myfile)
-            except Exception as e:
-                print("Could not save to file `{}`: e=`{}`".format(
-                    full_filename, e
-                ))
-
-        return my_data
+            my_data = None
+            #threshold_time = time.mktime(time.gmtime(60*60*8))   # 8 hours
+            threshold_time = 60*60*8
 
+            # Get the enlapsed seconds since last file update
+            diff_time = time.time() - osp.getmtime(full_filename)
+            # If the file was updated less than the threshold time, 
+            # it means that the file is still fresh, we can load it.
+            # Otherwise skip and return None to force a re-download
+            if threshold_time > diff_time:
+                # Load the JSON data
+                try:
+                    with open(full_filename,mode='r',encoding='utf-8') as myfile:
+                        #my_data = myfile.read()
+                        my_data = json.load(myfile)
+                except Exception as e:
+                    print("Could not save to file `{}`: e=`{}`".format(
+                        full_filename, e
+                    ))
+            return my_data
+        else:
+            return None
 
     def saveToFile(self, data_list: dict, filename: str) -> bool:
         """Save a dictionary to file
@@ -672,4 +826,4 @@ class XTream():
 
     def get_all_epg_URL(self):  
         URL = '%s/xmltv.php?username=%s&password=%s' % (self.server, self.username, self.password)  
-        return URL
\ No newline at end of file
+        return URL
